; --- ustawienia ---
.equ DS1302_Port	= PORTD
.equ DS1302_DDR		= DDRD
.equ DS1302_PIN		= PIND

.equ DS1302_CE		= 0
.equ DS1302_IO		= 1
.equ DS1302_SCLK	= 2
;-------------------

; --- pomocnicze ---
.def command_byte	= r16
.def data_byte		= r17
.def delay			= r18	; na pocztku naley zdefiniowa jaki wolny rejestr jako delay 
.def i				= r19	; na pocztku naley zdefiniowa jaki wolny rejestr jako i 
.def j				= r20	; na pocztku naley zdefiniowa jaki wolny rejestr jako j
;-------------------

; --- ustawienia danych ---
.;   0b[x]0000000
.equ CMD        = 0b10000000
 ;   0b0[x]000000
.equ RAM        = 0b01000000 
.equ CLOCK      = 0b00000000 
;   0b0000000[x]
.equ READ       = 0b00000001 
.equ WRITE      = 0b00000000 
;-------------------

; --- adresy --- 
;    (zegar)
; przesunite o 1 pozycj w lewo, poniewa na pozycji LSB 
; jest bit kierunku transmisji (R/!W) 
.equ ADDR_SEC	= (0x00<<1)
.equ ADDR_MIN	= (0x01<<1)
.equ ADDR_HRS	= (0x02<<1)
.equ ADDR_DATE	= (0x03<<1)
.equ ADDR_MONTH	= (0x04<<1)
.equ ADDR_DAY	= (0x05<<1)
.equ ADDR_YEAR	= (0x06<<1)
.equ ADDR_TCS	= (0x07<<1);TRICKLE CHARGER SELECT
; --- flagi --- 
.equ ADDR_CHFLAG	= (0x00<<1);CLOCK HALT (0b00[00000]0_
.equ ADDR_WPFLAG	= (0x07<<1);WRITE PROTECTION (0b00[00111]0)
; -------------

; --- adresy --- 
;  ( 31 x 8 RAM)
;[Scratchpad Data Storage]
; -  -  -  -  -
;   0b00[xxxxx]0
.equ ADDR_RAM1	 = (0x00<<1)
.equ ADDR_RAM2	 = (0x01<<1)
.equ ADDR_RAM3	 = (0x02<<1)
.equ ADDR_RAM4	 = (0x03<<1)
.equ ADDR_RAM5	 = (0x04<<1)
.equ ADDR_RAM6	 = (0x05<<1)
.equ ADDR_RAM7	 = (0x06<<1)
.equ ADDR_RAM8	 = (0x07<<1)
.equ ADDR_RAM9	 = (0x08<<1)
.equ ADDR_RAM10	 = (0x09<<1)
.equ ADDR_RAM11	 = (0x0A<<1)
.equ ADDR_RAM12	 = (0x0B<<1)
.equ ADDR_RAM13	 = (0x0C<<1)
.equ ADDR_RAM14	 = (0x0D<<1)
.equ ADDR_RAM15	 = (0x0E<<1)
.equ ADDR_RAM16	 = (0x0F<<1)
.equ ADDR_RAM17	 = (0x10<<1)
.equ ADDR_RAM18	 = (0x11<<1)
.equ ADDR_RAM19	 = (0x12<<1)
.equ ADDR_RAM20	 = (0x13<<1)
.equ ADDR_RAM21	 = (0x14<<1)
.equ ADDR_RAM22	 = (0x15<<1)
.equ ADDR_RAM23	 = (0x16<<1)
.equ ADDR_RAM24	 = (0x17<<1)
.equ ADDR_RAM25	 = (0x18<<1)
.equ ADDR_RAM26	 = (0x19<<1)
.equ ADDR_RAM27	 = (0x1A<<1)
.equ ADDR_RAM28	 = (0x1B<<1)
.equ ADDR_RAM29	 = (0x1C<<1)
.equ ADDR_RAM30	 = (0x1D<<1)
.equ ADDR_RAM31	 = (0x1E<<1)
;-------------------


;inicjacja lini-----
DS1302_INIT:

	cbi DS1302_Port,DS1302_SCLK		;LOW (stan niski) == cbi PORTB,PORTB2
	cbi DS1302_Port,DS1302_IO		;LOW (stan niski) == cbi PORTB,PORTB1
	cbi DS1302_Port,DS1302_CE		;LOW (stan niski) == cbi PORTB,PORTB0
	
	sbi DS1302_DDR,DS1302_SCLK		;output	 == sbi DDRB,DDB2
	sbi DS1302_DDR,DS1302_IO		;output	 == sbi DDRB,DDB1
	sbi DS1302_DDR,DS1302_CE		;output	 == sbi DDRB,DDB0
	
ret	
;-------------------

; =================================================== 
; proceduraa Write_BYTE wysyajca byte danych 
; do odpowiedniej komrki pamici/rejestru ukadu DS1302 
; wejcie: 
;    command_byte - bajt zawierajcy komend (CMD), 
;                   przeznaczenie danych (RAM|CLOCK), 
;                   kierunek transmisji (READ | WRITE), 
;                   adres komrki pamici/rejestru (ADDR_xxx) 
;    data_byte    - bajt zawierajcy dane do wysania 
; wyjcie: 
;    brak 
DS1302_WriteData:

	;zapamietaj rejestry
	push i
	push j

	;=============================================================

	; ****** #1 ****** 
	;rozpoczynam transmicje
		SBI DS1302_Port,DS1302_CE 

		; ****** #2 ****** 
		; 4us - 200ns = 3800ns 
		; 3800ns / 50ns = 76 taktw 
		LDI delay, 26
		delay_loop1:
		DEC delay        ; 1 takt 
		BRNE delay_loop1 ; 2 takty 
		                ; ----- 
		                ; 3 takty * 26 = 78 ( w przyblieniu) 
		; ****** #3 ******
		;8 bitow - COMMAND DATA 
		LDI i, 8
		send_byte_loop1: 

				; ****** #4 ****** 
				;ustaw linie IO
				SBRS   command_byte, 0       ; jeeli LSB w command_byte = 0 
				CBI DS1302_Port, DS1302_IO   ; linia DS1302_IO = 0 

				SBRC   command_byte, 0       ; jeeli LSB w command_byte = 1 
				SBI DS1302_Port, DS1302_IO   ; linia DS1302_IO = 1 

				; ****** #10 ****** 
				;ten bit juz wyslano
				LSR command_byte 


				; ****** #5 ****** 
				; dla pewnoci 5 taktw 
				nop 
				nop 
				nop 
				nop 
				nop 

				; ****** #6 ****** 
				;SCLK w stanie wysokim-DS1302 czyta dane
				SBI DS1302_Port, DS1302_SCLK 

					; ****** #7 ****** 
					; 1000ns / 50ns = 20 taktw 
					LDI delay, 7 
					delay_loop2: 
					DEC delay        ; 1 takt 
					BRNE delay_loop2 ; 2 takty 
					                ; ----- 
					                ; 3 takty * 7 = 21 ( w przyblieniu) 

				; ****** #8 ****** 
				;koniec zczytywania danych
				CBI DS1302_Port, DS1302_SCLK 

			; ****** #9 ****** 
			; 800ns / 50ns = 16 taktw 
			;przerwa miedzy danymi
			LDI delay, 6 
			delay_loop3: 
			DEC delay        ; 1 takt 
			BRNE delay_loop3 ; 2 takty 
			                ; ----- 
			                ; 3 takty * 6 = 18 ( w przyblieniu) 

		; ****** #11 ****** 
		;kontunuluj lub zakoncz petle COMMAND BYTE
		DEC i 
		BRNE send_byte_loop1 

	;=============================================================

		; ****** #12 ****** 
		; DATA byte
		LDI i, 8
		send_byte_loop2: 

			; ****** #13 ****** 
			;ustaw dane
			SBRS   data_byte, 0          ; jeeli LSB w data_byte = 0 
			CBI DS1302_Port, DS1302_IO   ; linia DS1302_IO = 0 

			SBRC   data_byte, 0          ; jeeli LSB w data_byte = 1 
			SBI DS1302_Port, DS1302_IO   ; linia DS1302_IO = 1 

			; ****** #14 ****** 
			;dla pewenosci 5 taktw
			nop 
			nop 
			nop 
			nop 
			nop 

				; ****** #15 ****** 
				;wyslij dane
				SBI DS1302_Port, DS1302_SCLK 

					; ****** #16 ****** 
					; 1000ns / 50ns = 20 taktw 
					LDI delay, 7 
					delay_loop4: 
					DEC delay        ; 1 takt 
					BRNE delay_loop4 ; 2 takty 
					                ; ----- 
					                ; 3 takty * 7 = 21 ( w przyblieniu) 
					
				; ****** #17 ****** 
				;koniec wysylania
				CBI DS1302_Port, DS1302_SCLK 

			; ****** #18 ****** 
			; 800ns / 50ns = 16 taktw 
			;przerwa miedzy kolejnymi taktami
			LDI delay, 6 
			delay_loop5: 
			DEC delay        ; 1 takt 
			BRNE delay_loop5 ; 2 takty 
			                ; ----- 
			                ; 3 takty * 6 = 18 ( w przyblieniu) 

			; ****** #19 ****** 
			LSR data_byte

		; ****** #20 ****** 
		DEC i 
		BRNE send_byte_loop2

	;==================================================


	; ****** #21 ****** 
	; koniec transmicji
	CBI DS1302_Port,DS1302_CE	;OFF

	;przywracamy rejestry
	pop j
	pop i
RET 

; ==== koniec Write_BYTE =================================


;odczyt danych
	;command_byte
	;data_byte
DS1302_ReadData:


	;zapamietaj rejestry
	push i
	push j
	;=============================================================

	; ****** #1 ****** 
	;rozpoczynam transmicje
	SBI DS1302_Port,DS1302_CE 

		; ****** #2 ****** 
		; 4us - 200ns = 3800ns 
		; 3800ns / 50ns = 76 taktw 
		LDI delay, 26
		RD_delay_loop1:
		DEC delay        ; 1 takt 
		BRNE RD_delay_loop1 ; 2 takty 
		                ; ----- 
		                ; 3 takty * 26 = 78 ( w przyblieniu) 

		; ****** #3 ******
		;8 bitow - COMMAND DATA 
		LDI i, 8
		RD_send_byte_loop1: 


				; ****** #4 ****** 
				;ustaw linie IO
				SBRS   command_byte, 0       ; jeeli LSB w command_byte = 0 
				CBI DS1302_Port, DS1302_IO   ; linia DS1302_IO = 0 

				SBRC   command_byte, 0       ; jeeli LSB w command_byte = 1 
				SBI DS1302_Port, DS1302_IO   ; linia DS1302_IO = 1 

				; ****** #5 ****** 
				;ten bit juz wyslano
				LSR command_byte 


				; ****** #6 ****** 
				; dla pewnoci 5 taktw 
				nop 
				nop 
				nop 
				nop 
				nop 

				; ****** #7 ****** 
				;SCLK w stanie wysokim-DS1302 czyta dane
				SBI DS1302_Port, DS1302_SCLK 

					; ****** #8 ****** 
					; 1000ns / 50ns = 20 taktw 
					LDI delay, 7 
					RD_delay_loop2: 
					DEC delay        ; 1 takt 
					BRNE RD_delay_loop2 ; 2 takty 
					                ; ----- 
					                ; 3 takty * 7 = 21 ( w przyblieniu) 

				; ****** #9' ****** 
				;przeskocz ten krok dla ostatniego bajtu
				ldi j,1
				CPSE i,j				
				CBI DS1302_Port, DS1302_SCLK 

			; ****** #10 ****** 
			; 800ns / 50ns = 16 taktw 
			;przerwa miedzy danymi
			LDI delay, 6 
			RD_delay_loop3: 
			DEC delay        ; 1 takt 
			BRNE RD_delay_loop3 ; 2 takty 
			                ; ----- 
			                ; 3 takty * 6 = 18 ( w przyblieniu) 

		; ****** #11 ****** 
		;kontunuluj lub zakoncz petle COMMAND BYTE
		DEC i 
		BRNE RD_send_byte_loop1 

	;=============================================================

		;tu bedziemy zapisywa dane
		clr data_byte;0b00000000

	    ; ****** #12 ********
		;ustaw I/O jako port WEJSCIA
		;DS1302 poda dane na opajacym zboczu SCLK
		cbi DS1302_DDR,DS1302_IO	; wcz tryb wejsciowy lini 1 portu B
			;to samo:  cbi DDRB, DDB1
		cbi DS1302_PORT,DS1302_IO   ; wycz wezntrzne podciganie linie 1 portu B
			;to samo:  cbi PORTB, PORTB1

		; ****** #13 ****** 
		; dla pewnoci 5 taktw 
		nop					; oczekuj na ustalenie stanu wejscia (1 cykl zegarowy)
		nop
		nop
		nop
		nop

		; ****** #14 ****** 
		; DATA byte
		LDI i, 8
		RD_send_byte_loop2: 

			; ****** #15 ****** 
				;czytaj dane
				CBI DS1302_Port, DS1302_SCLK 

					; ****** #16 ****** 
					; 1000ns / 50ns = 20 taktw 
					LDI delay, 7 
					RD_delay_loop4: 
					DEC delay        ; 1 takt 
					BRNE RD_delay_loop4 ; 2 takty 
					                ; ----- 
					                ; 3 takty * 7 = 21 ( w przyblieniu) 



					; ******** #16 ************
					;zapisz wyslane dane
					push j

						in j,DS1302_PIN; todo: zamien na DS1302_Port-??

						;czytaj
						;SBRS	j,DS1302_IO				;jeeli bit jest wyzerowany
						;CBR data_byte,7	;wyzeruj

						SBRC	j,DS1302_IO				;jeeli bit jest ustawiony
						SBR data_byte,0b10000000		;to go ustaw					;UWAGA: a nie: "SBR 	data_byte,7"

						;przesun bity w lewo(ten ju odczytano)
						;(przeskocz ten krok w ostatnim kroku)
						ldi j,1
						CPSE i,j				
						LSR data_byte
						;----------

					pop j
					;--------------


					

				; ****** #17 ****** 
				;koniec
				SBI DS1302_Port, DS1302_SCLK 

			; ****** #18 ****** 
			; 800ns / 50ns = 16 taktw 
			;przerwa miedzy kolejnymi taktami
			LDI delay, 6 
			RD_delay_loop5: 
			DEC delay        	; 1 takt 
			BRNE RD_delay_loop5 ; 2 takty 
			                ; ----- 
			                ; 3 takty * 6 = 18 ( w przyblieniu) 


		; ****** #19 ****** 
		DEC i 
		BRNE RD_send_byte_loop2

	;==================================================


	; ****** #21 ****** 
	; koniec transmicji
	CBI DS1302_Port,DS1302_CE	;OFF

	;zapamietaj rejestry
	pop j
	pop i
ret
;-------------------


;ustala czy czytamy czy zapisujemy
	;command_byte
	;data_byte
;jeeli BIT0 w r16 HI  - odczyt
;jeeli BIT0 w r16 LOW - zapis
DS1302_SendCode:

	rcall DS1302_Init

			SBRS	command_byte,0	;jezeli to komenda zapisu
			rcall DS1302_WriteData	;wywolaj procedure zapisu
			SBRC	command_byte,0	;jezeli to komenda odczytu
			rcall DS1302_ReadData	;wywolaj procedure odczytu

	rcall DS1302_Init		;koniec pracy (note take 'czysci' DS1302_CE)
ret
;-------------------
